"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var args = process.argv.splice(2);
var fileName = args[0];
console.log("Priting file: ", fileName, " args: ", args);
var ts = require("typescript");
var removeList = [
    "parent",
    "_children",
    "statements",
    "pos",
    "end",
    "modifierFlagsCache",
    "transformFlags",
    "flowNode",
    "parent",
];
function syntaxKindToName(kind) {
    return ts.SyntaxKind[kind];
}
exports.syntaxKindToName = syntaxKindToName;
function simplifyObject(checker, item, included) {
    if (included === void 0) { included = []; }
    if (typeof item !== "object") {
        return item;
    }
    var includedIdx = included.indexOf(item);
    if (includedIdx !== -1) {
        return "node:" + includedIdx;
    }
    included.push(item);
    var id = included.length - 1;
    var result = Object.getOwnPropertyNames(item)
        .reduce(function (acc, i) {
        var ignore = removeList.indexOf(i) !== -1;
        if (ignore) {
            //acc[i] = "---OMITTED---"
        }
        else if (i === "kind") {
            acc[i] = syntaxKindToName(item[i]);
        }
        else {
            acc[i] = simplifyObject(checker, item[i], included);
        }
        return acc;
    }, { id: id, symbol: null });
    var symbol = checker.getSymbolAtLocation(item);
    if (symbol != null) {
        result.symbol = {
            name: symbol.name,
        };
    }
    return result;
}
function log(title, items, customLog) {
    if (customLog === void 0) { customLog = null; }
    items = items.map(function (i) { return (__assign({}, i, { parent: undefined })); });
    console.log(title + " ==========================================");
    console.log(items);
    if (customLog) {
        console.log(title + " (custom) ==========================================");
        items.forEach(function (i) { return customLog(i); });
    }
}
function printMap(checker, sourceFile) {
    console.log(JSON.stringify(simplifyObject(checker, sourceFile), null, 2));
}
/** True if this is visible outside this file, false otherwise */
function isNodeExported(node) {
    // Parse the modifier array for the export keyword. If it is found
    // and the node.parent is a sourcefile, return true
    // This only returns top level exports
    var modifiers = node.modifiers;
    if (modifiers) {
        for (var i = 0; i < modifiers.length; i++) {
            if (modifiers[i].kind === ts.SyntaxKind.ExportKeyword) {
                return node.parent.kind === ts.SyntaxKind.SourceFile;
            }
        }
    }
    return false;
}
function symbolMapToString(map) {
    var values = [];
    map.forEach(function (value, key) {
        values.push(value.name);
    });
    return values.join("; ");
}
function simplePrint(checker, node, indent) {
    if (indent === void 0) { indent = 0; }
    var indentText = Array(indent * 4).join(" ");
    var info = [];
    function addSymbol(info, node, prefix) {
        if (prefix === void 0) { prefix = ""; }
        var s = checker.getSymbolAtLocation(node);
        if (!s) {
            return;
        }
        if (s.exports && s.exports.size > 0) {
            info.push(prefix + "exports: " + symbolMapToString(s.exports));
        }
        if (s.globalExports && s.globalExports.size > 0) {
            info.push(prefix + "globalExports: " + symbolMapToString(s.exports));
        }
        info.push(prefix + "name: " + s.name);
        var type = checker.getTypeOfSymbolAtLocation(s, node);
        if (type && type.symbol) {
            info.push(prefix + "type: " + type.symbol.name);
        }
        var comments = s.getDocumentationComment();
        if (comments.length > 0) {
            info.push(prefix + "comment: \"" + comments.map(function (i) { return i.text; }).join("; ") + "\"");
        }
    }
    if (node.kind === ts.SyntaxKind.FunctionDeclaration) {
        var d = node;
        info.push("name: " + d.name.text);
        info.push("type: " + d.type.getText());
    }
    if (node.kind === ts.SyntaxKind.ClassDeclaration) {
        var d = node;
        info.push("name: " + d.name.text);
        info.push("members: [" + d.members.map(function (i) { return i.name.getText(); }).join(", ") + "]");
    }
    if (node.kind === ts.SyntaxKind.Identifier) {
        var d = node;
        addSymbol(info, node);
    }
    if (node.kind === ts.SyntaxKind.InterfaceDeclaration) {
        var d = node;
        addSymbol(info, node);
        addSymbol(info, d.name, "name-");
    }
    if (node.kind === ts.SyntaxKind.ExportDeclaration) {
        var d = node;
        info.push("name: " + d.name);
        addSymbol(info, node);
    }
    if (node.kind === ts.SyntaxKind.ExportAssignment) {
        var d = node;
        info.push("name: " + d.name);
        addSymbol(info, node);
    }
    if (node.kind === ts.SyntaxKind.VariableDeclaration) {
        var d = node;
        addSymbol(info, d.name, "name-");
        addSymbol(info, node);
    }
    if (node.kind === ts.SyntaxKind.VariableStatement) {
        var d = node;
        if (d.declarationList && d.declarationList.declarations.length === 1) {
            var dec = d.declarationList.declarations[0];
            info.push("declarations: " + ("(kind: " + syntaxKindToName(dec.kind) + ", name: " + dec.name.getText() + ", name.kind: " + syntaxKindToName(dec.name.kind) + ")"));
            if (dec.type) {
                info.push("declaration.type: " + syntaxKindToName(dec.type.kind));
            }
            if (dec.initializer) {
                info.push("declaration.initializer: " + syntaxKindToName(dec.initializer.kind));
            }
        }
        addSymbol(info, node);
    }
    if (node.kind === ts.SyntaxKind.CallExpression) {
        var d = node;
        info.push("arguments: " + d.arguments.map(function (i) { return i.getText(); }).join(";"));
        if (d.contextualType) {
            info.push("contextualType: " + d.contextualType.symbol.name);
        }
    }
    if (isNodeExported(node)) {
        info.push("exported: true");
    }
    var infoText = info.length === 0 ? "" : "(" + info.join(", ") + ")";
    console.log("" + indentText + syntaxKindToName(node.kind) + " " + infoText);
    ts.forEachChild(node, function (child) {
        simplePrint(checker, child, indent + 1);
    });
}
var defaultOptions = {
    target: ts.ScriptTarget.Latest,
    module: ts.ModuleKind.CommonJS
};
var program = ts.createProgram([fileName], defaultOptions);
var sourceFile = program.getSourceFile(fileName);
var checker = program.getTypeChecker();
if (args.length === 1) {
    printMap(checker, sourceFile);
}
else if (args[1] === "simple") {
    simplePrint(checker, sourceFile);
}
//# sourceMappingURL=print.js.map